[data-md-color-scheme="default"],
:root {
  --bg: #fbf8f1;
  --paper: #fffdf9;
  --ink: #2b2421;
  --accent1: #c00000;
  --accent2: #9aa4b3;
  --text: #312925;
  --muted: #746a63;
  --paper-border: rgba(116, 106, 99, 0.16);
}

[data-md-color-scheme="default"] {
  --md-default-bg-color: var(--bg);
  --md-default-fg-color: var(--text);
  --md-default-fg-color--light: var(--muted);
  --md-primary-fg-color: var(--paper);
  --md-primary-fg-color--light: var(--bg);
  --md-primary-fg-color--dark: var(--paper);
  --md-primary-bg-color: var(--ink);
  --md-accent-fg-color: var(--accent1);
  --md-typeset-a-color: var(--accent1);
  --md-code-bg-color: var(--paper);
}

.md-header,
.md-tabs {
  background-color: var(--bg);
  border-bottom: 1px solid rgba(116, 106, 99, 0.18);
}

.md-search__form {
  background-color: var(--paper);
  border: 1px solid var(--paper-border);
  box-shadow: none;
}

.md-search__input,
.md-search__input::placeholder {
  color: var(--text);
}

.md-typeset h1,
.md-typeset h2,
.md-typeset h3 {
  color: var(--ink);
}

.md-typeset code {
  border: 1px solid var(--paper-border);
  border-radius: 4px;
}

.md-header__button.md-logo {
  align-items: center;
  display: inline-flex;
  line-height: 0;
  margin: 0.1rem 0.65rem 0.1rem 0.35rem;
  padding: 0;
  transition: none;
}

.md-header__button.md-logo:hover {
  opacity: 1;
}

.md-header__title .md-ellipsis {
  visibility: hidden;
}

.identity-wordmark {
  --identity-wordmark-letter-spacing: -0.03em;
  position: relative;
  display: inline-block;
  width: fit-content;
  line-height: 1;
  text-rendering: geometricPrecision;
  -webkit-font-smoothing: antialiased;
  -moz-osx-font-smoothing: grayscale;
}

.identity-wordmark .tri {
  position: relative;
  display: inline-block;
  font-family: Arial, Helvetica, sans-serif;
  font-size: var(--identity-wordmark-size, 56px);
  letter-spacing: var(--identity-wordmark-letter-spacing);
}

.identity-wordmark .layer {
  position: absolute;
  inset: 0;
  pointer-events: none;
}

.identity-wordmark .layer.red {
  color: var(--accent1);
  transform: translate(calc(-1 * var(--identity-wordmark-shift, 2px)), var(--identity-wordmark-shift, 2px));
  font-weight: 400;
}

.identity-wordmark .layer.blue {
  color: var(--accent2);
  transform: translate(var(--identity-wordmark-shift, 2px), calc(-1 * var(--identity-wordmark-shift, 2px)));
  font-weight: 400;
}

.identity-wordmark .layer.black {
  position: relative;
  color: var(--ink);
  font-weight: 400;
}

.identity-wordmark__sr-only {
  position: absolute;
  width: 1px;
  height: 1px;
  padding: 0;
  margin: -1px;
  overflow: hidden;
  clip: rect(0, 0, 0, 0);
  white-space: nowrap;
  border: 0;
}

.md-header__button.md-logo img {
  display: block;
  height: 2.4rem;
  width: auto;
}

.homepage-identity .identity-wordmark {
  --identity-wordmark-size: clamp(40px, 8vw, 62px);
  --identity-wordmark-shift: 1.5px;
  margin: 0;
}

.homepage-identity {
  margin: 0 0 2rem;
  max-width: 38rem;
}

.identity-tagline {
  max-width: 38rem;
  margin: 0.75rem 0 0;
  color: var(--text);
  font-size: 1rem;
  line-height: 1.6;
}

/* Keep the navigation and TOC panels scrollable without persistent scrollbar chrome. */
.md-sidebar__scrollwrap {
  -ms-overflow-style: none;
  scrollbar-width: none;
}

.md-sidebar__scrollwrap::-webkit-scrollbar {
  display: none;
}

@media screen and (max-width: 76.234375em) {
  .md-header__button.md-logo {
    display: none;
  }

  .md-header__title .md-ellipsis {
    visibility: visible;
  }
}

@media screen and (max-width: 44.984375em) {
  .homepage-identity .identity-wordmark {
    --identity-wordmark-size: clamp(34px, 10vw, 50px);
    --identity-wordmark-shift: 1px;
  }
}

/* Keep only the release tag in the repo metadata pill. */
.md-source__fact--stars,
.md-source__fact--forks {
  display: none;
}
